import qubtree
import QUB
import QUBB
import string
import re
import operator

class FieldAcceptException(Exception):
    def __init__(self, badStr, msg):
        self.badStr = badStr
        self.msg = msg
    def __str__(self):
        return self.msg # + ' (' + self.badStr + ')'

def acceptString(x):
    return x

def acceptStringValue() :
    return lambda x:acceptString(x)

def acceptInt(x):
    try:
        return int(x)
    except:
        raise FieldAcceptException(x, "must be an integer")

def acceptFloat(x):
    try:
        return float(x)
    except:
        raise FieldAcceptException(x, "must be a number")

def acceptCmp(convert, compare, errmsg):
    def accept(s):
        try:
            x = convert(s)
            if not compare(x):
                raise FieldAcceptException(s, errmsg)
            return x
        except ValueError:
            raise FieldAcceptException(s, errmsg)
    return accept

errmsg_cmp = lambda cmpname: lambda typename, bound: "must be %s %s %s" % (typename, cmpname, bound)
errmsg_gt = errmsg_cmp("greater than")
errmsg_ge = errmsg_cmp("greater than or equal to")
errmsg_lt = errmsg_cmp("less than")
errmsg_le = errmsg_cmp("less than or equal to")
name_int = "an integer"
name_float = "a number"

def acceptIntGreaterThan(lb):
    return acceptCmp(int, lambda x:x>lb, errmsg_gt(name_int, lb))
def acceptIntGreaterThanOrEqualTo(lb):
    return acceptCmp(int, lambda x:x>=lb, errmsg_ge(name_int, lb))
def acceptIntLessThan(ub):
    return acceptCmp(int, lambda x:x<ub, errmsg_lt(name_int, ub))
def acceptIntLessThanOrEqualTo(ub):
    return acceptCmp(int, lambda x:x<=ub, errmsg_le(name_int, ub))
def acceptIntBetween(lb, ub):
    return acceptCmp(int, lambda x:lb<=x<=ub, "must be %s between %s and %s" % (name_int, lb, ub))
def acceptFloatGreaterThan(lb):
    return acceptCmp(float, lambda x:x>lb, errmsg_gt(name_float, lb))
def acceptFloatGreaterThanOrEqualTo(lb):
    return acceptCmp(float, lambda x:x>=lb, errmsg_ge(name_float, lb))
def acceptFloatLessThan(ub):
    return acceptCmp(float, lambda x:x<ub, errmsg_lt(name_float, ub))
def acceptFloatLessThanOrEqualTo(ub):
    return acceptCmp(float, lambda x:x<=ub, errmsg_le(name_float, ub))
def acceptFloatBetween(lb, ub):
    return acceptCmp(float, lambda x:lb<=x<=ub, "must be %s between %s and %s" % (name_float, lb, ub))
def acceptFloatAbsBetween(lb, ub):
    return acceptCmp(float, lambda x:(lb<=x<=ub) or ((-ub)<=x<=(-lb)), "must be %s between +/- %s and %s" % (name_float, lb, ub))


# eats all the spaces commas and dashes, so don't accept on those
def acceptAcceptList(s, accept, dashrange, canDash, description):
    "e.g. '1, 2, 3-5 7' -> [1, 2, 3, 4, 5, 7]"

    if s == '':
        return []

    if canDash:    
        # replace \d+ ?- ?\d+ with \d+-\d+
        s = re.sub(r'([^ eE-]+) ?- ?([^ -]+)', r'\1-\2', s)
    # split along [, ]+
    tokens = re.split(r'[ ,\t]+', s)
    # foreach token:  if \d+ append int; elif \d+-\d+ append ints
    result = []
    try:
        for tok in tokens:
            if canDash:
                match = re.search(r'([^ eE-]+)-([^ -]+)', tok)
                if match:
                    a, b = accept(match.group(1)), accept(match.group(2))
                    result[len(result):] = dashrange(a, b)
                else:
                    result.append( accept(tok) )
            else:
                result.append( accept(tok) )
    except FieldAcceptException:
        raise FieldAcceptException(s, 'must be a list of %s, separated by ","%s' % (description, canDash and ' and "-"' or ''))
    return result

def acceptList(acceptor, dashrange, canDash, description):
    return lambda x: acceptAcceptList(x, acceptor, dashrange, canDash, description)

def acceptIntList(acceptor=acceptInt, description='integers'):
    def dashrange(i, j):
        if i <= j:
            return xrange(i, j+1)
        else:
            return xrange(i, j-1, -1)
    return acceptList(acceptor, dashrange, True, description)

def acceptFloatList(acceptor=acceptFloat, description='numbers'):
    def dashrange(i, j):
        raise FieldAcceptException('%f-%f', 'How shall I enumerate a continuous range?')
    return acceptList(acceptor, dashrange, False, description)

def acceptDimList(acceptList, dim, msg):
    def checkDim(lst):
        if dim != len(lst):
            raise FieldAcceptException(s, msg)
        return lst
    return lambda x: checkDim( acceptList(x) )

def SpacedNodeData(node):
    if len(node.data) == 0:
        return ''
    return string.join(map(str, node.data[:]), ' ')

def SetNodeValue(node, ix, value):
    try:
        len(value) # detect sequences, ignore index
        node.data = value
    except:
        if len(node.data) == 0:
            node.data = value #initialize with proper type
        if len(node.data) < (ix + 1):
            node.data.resize(ix + 1)
        node.data[ix] = value

def NodeSetter(value):
    return lambda Node, ix: SetNodeValue(Node, ix, value)

def NodeAcceptor(accept, path):
    getNode = eval('lambda root: root'+path)
    def NodeAccept(x, root, ix):
        # QUB.Report('%s = %s' % (path, str(x)))
        node = getNode(root)
        data = accept(x)
        SetNodeValue(node, ix, data)
    return NodeAccept

def NodeFormatFunc(format):
    def DoFormat(node, ix):
        data = node.data
        if not data:
            return ''
        elif data.type == qubtree.QTR_TYPE_STRING:
            return format(str(data)[:data.count])
        else:
            return format(data[ix])
    return DoFormat

def NodeFormat(format):
    return NodeFormatFunc(lambda val: (format % val))


#-----------------------------------------------

class DialogField:
    def __init__(self, *args, **kw):
        for key in kw.keys():
            self.__dict__[key] = kw[key]

class DialogLabel(DialogField):
    def __init__(self, caption='', name=''):
        DialogField.__init__(self, type='label', name=name, caption=caption)

class DialogHidden(DialogField):
    def __init__(self, name, value):
        DialogField.__init__(self, type='hidden', name=name, value=value)

class DialogText(DialogField):
    def __init__(self, name, caption, value='', acceptor=acceptString, format=str):
        DialogField.__init__(self, type='textbox', name=name, caption=caption, value=value,
                             acceptor=acceptor, format=format,
                             nodeAcceptor=NodeAcceptor(acceptor, '["%s"]'%name),
                             nodeFormat=NodeFormatFunc(format))

class DialogCheckBox(DialogField):
    def __init__(self, name, caption, value=0):
        DialogField.__init__(self, type='checkbox', name=name, caption=caption, value=value)

class DialogRadioGroup(DialogField):
    def __init__(self, name, caption, choices, value=''):
        DialogField.__init__(self, type='radiogroup', name=name, caption=caption, choices=choices, value=value)

